#!/bin/sh
#
# Dialogue with the user to see which types of tests they want
# to run.  
#

#
# yn ()
#
# Ask a y/n question.  Needs to have QUESTION and DEFAULT_ANS defined
# prior to call.  DEFAULT_ANS must be one of "yes", "no".  Returns 
# answer as "yes", "no" or "default" in 
# ANSWER_YN.
#
yn () {
    ANSWER="foo"
    until [ -z $ANSWER ] || [ $ANSWER = "yes" ] || [ $ANSWER = "y" ] || \
	  [ $ANSWER = "no" ] || [ $ANSWER = "n" ]  ; do {
		echo -n $QUESTION "(yes, no)" "["$DEFAULT_ANS"] "
		read ANSWER
    } ; done

    if [ -z $ANSWER ] ; then {
	ANSWER_YN=$DEFAULT_ANS
    } ; elif [ $ANSWER = "yes" ] || [ $ANSWER = "y" ] ; then {
	ANSWER_YN="yes"
    } ; else {
	ANSWER_YN="no"
    } ; fi
}

#
# The Cerberus test control file
#
echo '
This is the kernel-stress test front-end.

To run all the tests, you will need an empty SCSI partition and
an empty IDE partition.  The test will overwrite any data in these
partitions.  Not to worry, you will have a chance to decline these
tests.

'


CTCS_DRIVER_FILE="/usr/local/bin/ctcs/RH-test.tcf"

#
# Flags indicate which tests we want to run
#
NFS_COMPILE="no"
TTCP="no"
IDE="no"
SCSI="no"
MEMORY="no"
FIFO_MMAP="no"

#
# SCSI dialogue
#
QUESTION="Do you want to run the destructive SCSI tests?"; DEFAULT_ANS="no"
yn
SCSI=$ANSWER_YN
if [ $SCSI = "yes" ] ; then {
    echo "Enter name of a SCSI partition that can be overwritten."
    echo -n "   (You can just type <return> here to skip this test.) "
    read SCSI_PARTITION
    if [ -z $SCSI_PARTITION ] ; then {
	SCSI="no"
	echo "SCSI test will be skipped."
    } ; else {
	echo "SCSI test will write to" $SCSI_PARTITION"."
    } ; fi
} ; else {
    echo "SCSI test will be skipped."
} ; fi


#
# TTCP dialogue
#
QUESTION="Do you want to test TCP/IP?"; DEFAULT_ANS="yes"; yn
TTCP=$ANSWER_YN
if [ $TTCP = "yes" ] ; then {
    echo "Enter hostname or IP number of this host:"
    echo -n "  (You can just type <return> here to skip this test.) "
    read LOCAL_HOST
    if [ -z $LOCAL_HOST ] ; then {
	echo "TCP/IP test will be skipped.";
	TTCP="no"
    } ; else {
	    echo "Enter hostname or IP number of remote host:"
	    echo -n "  (You can just type <return> here to skip this test.) "
	    read REMOTE_HOST
	    if [ -z $REMOTE_HOST ] ; then {
		echo "TCP/IP test will be skipped.";
		TTCP="no"
	    } ; else {
		echo "TCP/IP will run between" $LOCAL_HOST "and" $REMOTE_HOST"."
	    } ; fi
    } ; fi
} ; else {
   echo "TCP/IP test will be skipped."
} ; fi


QUESTION="Do you want to run the NFS-compile test?"; DEFAULT_ANS="yes"; yn
NFS_COMPILE=$ANSWER_YN
if [ $NFS_COMPILE = "yes" ] ; then {
    echo "NFS compile will be run."
} ; else {
    echo "NFS compile will be skipped."
} ; fi


#
# IDE dialogue
#
QUESTION="Do you want to run the destructive IDE tests?"; DEFAULT_ANS="no"; yn
IDE=$ANSWER_YN
if [ $IDE = "yes" ] ; then {
    echo "Enter name of an IDE partition that can be overwritten."
    echo -n "   (You can just type <return> here to skip this test.) "
    read IDE_PARTITION
    if [ -z $IDE_PARTITION ] ; then {
	IDE="no"
	echo "IDE test will be skipped."
    } ; else {
	echo "IDE test will write to" $IDE_PARTITION"."
    } ; fi
} ; else {
    echo "IDE test will be skipped."
} ; fi


QUESTION="Do you want to run memory tests?"; DEFAULT_ANS="yes"; yn
MEMORY=$ANSWER_YN
if [ $MEMORY = "yes" ] ; then {
    echo "Memory test will be run."
} ; else {
    echo "Memory test will be skipped."
} ; fi


QUESTION="Do you want to test FIFO's and mmap?"; DEFAULT_ANS="yes"; yn
FIFO_MMAP=$ANSWER_YN
if [ $FIFO_MMAP = "yes" ] ; then {
    echo "FIFO/mmap test will be run."
} ; else {
    echo "FIFO/mmap test will be skipped."
} ; fi
	    

# 
# Create the driver file
#
rm -rf $CTCS_DRIVER_FILE >> /dev/null 2>&1
cat > $CTCS_DRIVER_FILE <<EOF
#
# This is the ctcs driver file.  This file is auto-created.
#
set verbose 1
EOF

if [ $SCSI = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE <<EOF
bg 2 SCSI-TEST iotest30_driver.sh $SCSI_PARTITION
EOF
} ; fi

if [ $NFS_COMPILE = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE <<EOF
bg 4 NFS-COMPILE nfstest.sh
EOF
} ; fi

if [ $TTCP = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 1024 TTCP ttcp_driver.sh $LOCAL_HOST $REMOTE_HOST
EOF
} ; fi

if [ $IDE = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 8 IDE blockrdtst $IDE_PARTITION
EOF
} ; fi


if [ $MEMORY = "yes" ] ; then {
#
# Kludge, I know.  Don't start on me.
#
    PHYSICAL_MEMORY=`wc -c /dev/mem | cut -d"/" -f1`
    PHYSICAL_MEMORY_MB=`expr $PHYSICAL_MEMORY / 1024 / 1024`
    MEM_CEILING=`expr $PHYSICAL_MEMORY_MB \* 11 / 10` #110%
#
# memtst <freevm> <ceiling>
# where <freevm> = MB of VM to leave free
#       <ceiling> = but don't use more than this much memory
#
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 64 MEMORY memtst 64 $MEM_CEILING
EOF
} ; fi


if [ $FIFO_MMAP = "yes" ] ; then {
    cat >> $CTCS_DRIVER_FILE<<EOF
bg 256 FIFOS-MMAP dt_driver.sh
EOF
} ; fi

cat >> $CTCS_DRIVER_FILE<<EOF
wait
exit
EOF

echo; echo; echo "I'm about the run the following tests:"
if [ $SCSI = "yes" ] ; then { echo "  SCSI" } ; fi
if [ $TTCP = "yes" ] ; then { echo "  TCP/IP" } ; fi
if [ $NFS_COMPILE = "yes" ] ; then { echo "  NFS compile" } ; fi
if [ $IDE = "yes" ] ; then { echo "  IDE" } ; fi
if [ $MEMORY = "yes" ] ; then { echo "  Memory" } ; fi
if [ $FIFO_MMAP = "yes" ] ; then { echo "  FIFO's/mmap" } ; fi
echo

QUESTION="Proceed?"; DEFAULT_ANS="yes"; yn
PROCEED=$ANSWER_YN
if [ $PROCEED = "no" ] ; then {
    exit 0
} ; fi

cd $CTCS_DRIVER_FILE/..
echo "

OK, I'm ready to run the tests.  You can bail out at any point 
with <ctrl>-c.
"

exec ./run RH-test.tcf
