# cleanup is called right before we exit.  ERROR_VALUE tells us
# what we need to back out of.
#

# Restore nfs to it's state prior to execution of this script
restore_nfs_state () {
    if [ $NFS_STATUS = "reload" ] ; then # NFS was running
	/etc/rc.d/init.d/nfs reload >> /dev/null 2>&1
    elif [ $NFS_STATUS = "start" ] ; then # NFS was not running
	/etc/rc.d/init.d/nfs stop >> /dev/null 2>&1
    else                                  # Can't happen
       echo "Can't determine original NFS state."
    fi
}

# Restore /etc/exports to its state, if it existed.  If not, delete it.
restore_etc_exports () {
    if [ $RESTORE_EXPORTS_FILE = "yes" ] ; then {
	rm -f $EXPORTS_FILE
	mv $EXPORTS_FILE_BACKUP $EXPORTS_FILE
    } ;
    elif [ $RESTORE_EXPORTS_FILE = "no" ] ; then
	rm -f $EXPORTS_FILE
    else # Can't happen
	echo "Can't determine original state of /etc/exports."
    fi
}

# Get rid of any temp dirs we may have created.  Not all of those might exist.
remove_tmp_dirs () {
    rm -r $SOURCE1 $TARGET0 $TARGET1 >> /dev/null 2>&1
}

# 
# Depending on how far through the script we got, we might have to 
# perform multiple steps to restore the previous state.  This function
# does the necesary steps
#
function cleanup () {
case "$ERROR_VALUE"
in
#
# 0 means success, just back out of all changes

    0 ) rm -r $TARGET1 >> /dev/null 2>&1
	umount $TARGET1 >> /dev/null 2>&1
	umount $TARGET0 >> /dev/null 2>&1
        remove_tmp_dirs
	restore_etc_exports
	restore_nfs_state ;;

# 1,2 = error on startup, nothing to clean up.
    [12] ) ;;

# 3 = failed to make mount point directory
    3 ) restore_etc_exports ;;

# 4 = can't make temp dirs
# 5 = can't get probe NFS status
# 6 = bad NFS status
# 7 = can't modify NFS state
    [4567] ) restore_etc_exports
	    remove_tmp_dirs ;;

# 8 = can't mount on TARGET0
    8 ) restore_etc_exports
	restore_nfs_state
	remove_tmp_dirs ;;

# 9 = can't mount on TARGET1
    9 ) umount $TARGET0 >> /dev/null 2>&1
	restore_etc_exports
	restore_nfs_state
	remove_tmp_dirs ;;

# 10 = error reading from NFS mounted dir
    10 ) umount $TARGET0 >> /dev/null 2>&1
         umount $TARGET1 >> /dev/null 2>&1
	 restore_etc_exports
	 restore_nfs_state
	 remove_tmp_dirs ;;

# 11 = Can't write from client to server
# 12 = Error reading from client
    1[12] ) rm -rf $TARGET1 >> /dev/null 2>&1
        umount $TARGET0 >> /dev/null 2>&1
	umount $TARGET1 >> /dev/null 2>&1
	restore_etc_exports
	restore_nfs_state
	remove_tmp_dirs ;;

# anything else = bug in script
    * ) echo "Cleanup called with unknown error value.  This code is buggy." ;;

esac
}
