/* Copyright (c) 1996, 1997, 1998, 1999 by Solid Data Systems.
   *
   * Permission to use, copy, modify, and distribute this software for any
   * purpose with or without fee is hereby granted, provided that the above
   * copyright notice and this permission notice appear in all copies.
   *
   * THE SOFTWARE IS PROVIDED "AS IS" AND SOLID DATA SYSTEMS DISCLAIMS
   * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
   * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL SOLID DATA
   * SYSTEMS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
   * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
   * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
   * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
   * SOFTWARE. */

/**************************************************/
/* exercise.c  version 2.30                       */
/* INPUT:                                         */
/*   fd        file descriptor for device to test */
/*   blocks    # of blocks on the device to test  */
/* Richard Cooper 11/19/97                        */
/**************************************************/

#include "IOtest.h"

extern int do_log, skip_block_zero, min_xfer_size, max_xfer_size;
extern char rawdevice[20];
extern void current_time_to_log(void);

void exercise(int fd, long long blocks)
{
    static pid_t write_pid, read_pid;
    static int write_iterations, read_iterations;
    int i, iw = 0, ir = 0;
    long size;
    static long read_block_count = 0, write_block_count = 0;
    static long int charsread, charswrote;
    static char wbuffer[MAX_TRANSFER_SIZE];
    static char rbuffer[MAX_TRANSFER_SIZE];
    static long long random_block;
    static float tmp_block, tmp_block2;
    static long max_xfer, min_xfer;	/* transfer sizes in blocks */

#if DEBUG
    long write_min_start, write_max_start, read_min_start, read_max_start;
    long write_min_size, write_max_size, read_min_size, read_max_size;
#endif

    min_xfer_size = (min_xfer_size > 0) ? min_xfer_size : MIN_TRANSFER_SIZE;
    max_xfer_size = (max_xfer_size > 0) ? max_xfer_size : MAX_TRANSFER_SIZE;
    if (do_log == 1) {
	sprintf(buf, "Starting exerciser on device %s at ", rawdevice);
	logit(buf);
	current_time_to_log();
    }
    write_iterations = EXER_ITERATIONS;
    read_iterations = EXER_ITERATIONS;
    max_xfer = max_xfer_size / 512;
    min_xfer = min_xfer_size / 512;

#if DEBUG
    printf("initial write_iterations = %d\n", write_iterations);
    printf("initial read_iterations = %d\n", read_iterations);
#endif
    blocks = blocks - 1;	/* Turn # of blocks into highest block # */

#ifndef NO_BUFFER
    blocks = blocks - 3000;
#endif

/* Make sure we don't write off end... */
    blocks = blocks - MAX_TRANSFER_SIZE / 512;
    if (((LONG_MAX / 512) - blocks) < (MAX_TRANSFER_SIZE / 512)) {
	blocks = blocks - MAX_TRANSFER_SIZE / 512;
    }
/********************************************/
/* Fill the write buffer with random data   */
/********************************************/
    for (i = 0; i < (MAX_TRANSFER_SIZE); ++i) {
	wbuffer[i] = (char) rand();
    }

/****************************************************************************/
/* Adjust iteration counts depending on the average transfer size.          */
/* This should result in ABOUT 500,000 blocks being read or written during  */
/* each pass.                                                               */
/****************************************************************************/
    write_iterations = ((50 * write_iterations)) / ((MAX_TRANSFER_SIZE / 512 + MIN_TRANSFER_SIZE / 512) / 2);
    read_iterations = ((50 * read_iterations)) / ((MAX_TRANSFER_SIZE / 512 + MIN_TRANSFER_SIZE / 512) / 2);

#if DEBUG
    write_min_size = MIN_TRANSFER_SIZE + 1;
    write_max_size = MAX_TRANSFER_SIZE - 1;
    read_min_size = MIN_TRANSFER_SIZE + 1;
    read_max_size = MAX_TRANSFER_SIZE - 1;
    write_min_start = 10000;
    write_max_start = blocks - 10000;
    read_min_start = 10000;
    read_max_start = blocks - 10000;
#endif

/**********************************************/
/* Create a child process to do random writes */
/**********************************************/
    if ((write_pid = fork()) < 0) {
	io_error("error creating child process to do writes");
	exit(EXIT_FAILURE);
    }
#ifdef DEBUG
    if (write_pid != 0)
	printf("write process pid = %d\n", write_pid);
#endif

/****************************************************/
/* If write_pid = 0, this must be the write process */
/****************************************************/
    if (write_pid == 0) {

#if DEBUG
	printf("write .. blocks = %lld\n", blocks);
	printf("write_iterations = %d\n", write_iterations);
#endif

	srand((unsigned int) time(NULL));
	while (iw < write_iterations) {
	    tmp_block = (float) rand();
	    random_block = ((tmp_block / RAND_MAX) * (blocks - 1) + skip_block_zero);

#if DEBUG
	    if (write_min_start > random_block)
		write_min_start = random_block;
	    if (write_max_start < random_block)
		write_max_start = random_block;
#endif

/**********************************************************************/
/* This next line generates a random byte count size for the transfer */
/* between 512 and MAX_TRANSFER_SIZE bytes                            */
/**********************************************************************/
	    if (min_xfer_size != max_xfer_size) {

		size = 512 * ((rand() % (max_xfer - min_xfer)) + 1) + min_xfer_size;
	    } else {
		size = max_xfer_size;
	    }

#if DEBUG
	    if (write_min_size > size)
		write_min_size = size;
	    if (write_max_size < size)
		write_max_size = size;
#endif
#ifdef _LARGEFILE64_SOURCE
	    if ((lseek64(fd, (off64_t) (random_block * 512), SEEK_SET)) < 0) {
		perror("IOtest: lseek64");
		printf("Seek didn't go to block %lld (on write)\n", (random_block * 512));
		exit(EXIT_FAILURE);
	    }
#else
	    if ((lseek(fd, (random_block * 512), SEEK_SET)) < 0) {
		perror("IOtest: lseek");
		printf("Seek didn't go to block %lld (on write)\n", (random_block * 512));
		exit(EXIT_FAILURE);
	    }
#endif
	    if ((charswrote = (long int) write(fd, wbuffer, (size_t) size)) < 0) {
		printf("Error during transfer of %ld bytes starting at block %Lu\n", size, random_block);
		perror("IOtest:  write");
	    }
	    write_block_count = write_block_count + (size / 512);
	    iw++;
	}
	sprintf(buf, "%ld blocks written\n", write_block_count);
	printf("%s", buf);
	if (do_log == 1)
	    logit(buf);

#if DEBUG
	printf("min starting block# = %ld    max starting block# = %ld \n", write_min_start, write_max_start);
	printf("min xfer size = %ld blocks    max xfer size = %ld blocks\n", write_min_size / 512, write_max_size / 512);
#endif

	exit(EXIT_SUCCESS);
    }
/*******************************************************/
/* Create a child process to do random reads           */
/*******************************************************/
    if ((read_pid = fork()) < 0) {
	io_error("error creating child process to do reads");
	exit(EXIT_FAILURE);
    }
#ifdef DEBUG
    if (read_pid != 0)
	printf("read process pid = %d\n", read_pid);
#endif

/**************************************************/
/* If read_pid = 0, this must be the read process */
/**************************************************/
    if (read_pid == 0) {
#if DEBUG
	printf("read .. blocks = %lld\n", blocks);
	printf("read_iterations = %d\n", read_iterations);
#endif
	srand((unsigned int) time(NULL) + 1);	/* seed the random number generator */
	while (ir < read_iterations) {
	    /*random_block = rand() % blocks + 1; */
	    tmp_block2 = (float) rand();
	    random_block = (tmp_block2 / RAND_MAX) * (blocks - 1);

#if DEBUG
	    if (read_min_start > random_block)
		read_min_start = random_block;
	    if (read_max_start < random_block)
		read_max_start = random_block;
#endif

/**********************************************************************/
/* This next line generates a random byte count size for the transfer */
/* between 512 and MAX_TRANSFER_SIZ bytes                             */
/**********************************************************************/
	    if (min_xfer_size != max_xfer_size) {
		size = 512 * ((rand() % (max_xfer - min_xfer)) + 1) + min_xfer_size;
	    } else {
		size = max_xfer_size;
	    }

#if DEBUG
	    if (read_min_size > size)
		read_min_size = size;
	    if (read_max_size < size)
		read_max_size = size;
#endif

#ifdef _LARGEFILE64_SOURCE
	    if ((lseek64(fd, (off64_t) (random_block * 512), SEEK_SET)) < 0) {
		perror("IOtest: lseek64");
		printf("Seek didn't go to block %lld (on read)\n", (random_block * 512));
		exit(EXIT_FAILURE);
	    }
#else
	    if ((lseek(fd, (random_block * 512), SEEK_SET)) < 0) {
		perror("IOtest: lseek");
		printf("Seek didn't go to block %lld (on read)\n", (random_block * 512));
		exit(EXIT_FAILURE);
	    }
#endif
	    if ((charsread = (long int) read(fd, rbuffer, (size_t) size)) < 0) {
		printf("Error during transfer of %ld bytes starting at block %Lu\n", size, random_block);
/*              io_error("read error"); */
		perror("IOtest: read");
	    }
	    read_block_count = read_block_count + (size / 512);
	    ir++;
	}
	sprintf(buf, "%ld blocks read\n", read_block_count);
	printf("%s", buf);
	if (do_log == 1)
	    logit(buf);
#if DEBUG
	printf("min starting block# = %ld    max starting block# = %ld \n", read_min_start, read_max_start);
	printf("min xfer size = %ld blocks    max xfer size = %ld blocks\n\n", read_min_size / 512, read_max_size / 512);
#endif

	exit(EXIT_SUCCESS);
    }
/*******************************************************/
/* Wait for two child processes to finish, then return */
/*******************************************************/
    for (i = 1; i <= 2; ++i) {
	pid_t child_pid;
	child_pid = wait(NULL);
    }
}
