#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <math.h>
#include <malloc.h>
#include "memory.h"

void memtst(int verbose, int free_vm, int ceiling) {
  struct memory mem;
/*  int fd; */
  unsigned long nint;
  int *buf, *nbuf, i, align, block, extra_vm, j, k, l, ierror;
  int check[10];
  int consistent=0;
  int inconsistent=0;
  int nonerror=0;
  /* Get size of main memory */
  meminfo(&mem);

  /*
   * Malloc a region the size of main memory, or as large as we can get.
   * If the system has enough swap space, this will let other programs
   * continue to run while we do this test.  Also, it appears to be
   * a large enough region to exercise most of main memory.
   * Leave at least free_vm MB of free virtual memory for other burn-in tests.
   * If the swap space is less than free_vm, then this means we decrease
   * the size of the buffer to leave more free memory.
   */
  extra_vm = (free_vm * 1024 * 1024 - mem.total_swap);
  if(extra_vm > 0) {
    nint = (unsigned) ((mem.total_mem - extra_vm)/sizeof(int));
  } else {
    nint = (unsigned) ((mem.total_mem - 4*1024*1024)/sizeof(int));
  }
  if(nint*sizeof(int) > ceiling*1024*1024) {
	nint=(ceiling*1024*1024)/sizeof(int);
  }
  while((buf = (int *) calloc(sizeof(int),nint)) == NULL) {
    if(verbose) {
      printf("Unable to allocated requested memory.\n");
      printf("Decreasing request size by 5%%.\n");
    }
    nint = nint*0.95;
    if(nint < (.1*mem.total_mem)) {
      fprintf(stderr, "Can't get enough memory to be worthwhile.  Giving up.\n");
      exit(2);
    }
  }
  printf("Ceiling: %dK\n", (ceiling*1024));
  printf("Testing: %ld integers, %ldK", nint, nint * (long) sizeof(int)/1024);
  /*
   * There are 2 things we vary as we walk across memory: the alignment
   * and the block size.
   * The alignment varies from 0 to 3.  We write 4 byte words on non-aligned byte
   * boundaries hoping to detect any errors caused by crossing non-aligned
   * boundaries.  The block size is the number of bytes that we write before
   * doing a verifying read.  It varies from 256K to the entire
   * buffer size.  This exercises cache more with the small block sizes and
   * main memory more with larger block sizes.
   */
  for(align = 0; align < sizeof(int) ; align++) {
    nbuf = (int *) (&(buf[align]));
    for(block = (256*1024/4); block < (nint - sizeof(int)); block += (nint/10)) {
      /* Write to the buffer */
      if(verbose) printf("Writing block size %d (%dK) with alignment %d...", block, block*sizeof(int)/1024, align);
      fflush(stdout);
      for(i = 0; i < block; i++) {
	nbuf[i] = i + 0xaaaaaaaa;
      }
      /* Verify the result */
      if(verbose) printf("Verifying...");
      fflush(stdout);
      for(i = 0; i < block; i++) {
	if(nbuf[i] != (i + 0xaaaaaaaa)) {
	  fprintf(stderr, "Memory error at offset %d of %d : expected %x, got %x\n", i, block, i + 0xaaaaaaaa, nbuf[i]);
	  ierror=i;
	  fprintf(stderr, "Context: \n");
	  fprintf(stderr, "offset\texpected\tgot\t\n");
	  for(l=ierror-3 ; l < block && l < ierror+7 ; ++l) {
		if (l == ierror) {
			fprintf(stderr, "%d\t%x\t%x  *** fail location\n",l,0xaaaaaaaa+l,nbuf[l]);
		} else {
			fprintf(stderr, "%d\t%x\t%x\n",l,0xaaaaaaaa+l,nbuf[l]);
		}
	  }
	  fprintf(stderr, "Checking for consistency:\n");
	  for(j=0; j < 10 ; j++) {
		/* I could use check[j] as the iterator but I want to make
		   sure assembly version of the loop is identical to the
		   above one */
	    check[j] = -1;
	    for(k=0; k < block; k++) {
	      if(nbuf[k] != (k + 0xaaaaaaaa)) {
	        fprintf(stderr, "Check %d error at offset %d of %d: expected %x, got %x\n",j,k,block,k+0xaaaaaaaa,nbuf[k]);
	  ierror=k;
	  fprintf(stderr, "Context: \n");
	  fprintf(stderr, "offset\texpected\tgot\t\n");
	  for(l=ierror-3 ; l < block && l < ierror+7 ; ++l) {
		if (l == ierror) {
			fprintf(stderr, "%d\t%x\t%x  *** fail location\n",l,0xaaaaaaaa+l,nbuf[l]);
		} else {
			fprintf(stderr, "%d\t%x\t%x\n",l,0xaaaaaaaa+l,nbuf[l]);
		}
	  }
  	        check[j] = k;
	        break;
	      }
            }
          }
	  fflush(stdout);
	  fflush(stderr);
	  exit(1);
	}
      }
      if (verbose) printf("Done.\n");
      fflush(stdout);
    }
  }
  printf(" OK.\n");
  free(buf);
}
